@@ -5,7 +5,7 @@
     WWW:           http://www.swi-prolog.org
     Copyright (c)  2019-2022, University of Amsterdam
                               VU University Amsterdam
-		              CWI, Amsterdam
+			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
@@ -52,6 +52,10 @@ Event interface
 
 static void	free_event_callback(event_callback *cb);
 
+typedef int (*ev_func0)(void *);
+typedef int (*ev_func1)(void *, term_t);
+typedef int (*ev_func2)(void *, term_t, term_t);
+
 #ifdef O_PLMT
 #define INIT_LIST_LOCK(l) recursiveMutexInit(&(l)->lock)
 #define DELETE_LIST_LOCK(l) recursiveMutexDelete(&(l)->lock)
@@ -79,7 +83,7 @@ const event_type PL_events[] =
   GEVENT(PLEV_RETRACTNOBREAK,   ATOM_break,            3, onbreak),
   GEVENT(PLEV_FRAMEFINISHED,    ATOM_frame_finished,   1, onframefinish),
   GEVENT(PLEV_UNTABLE,		ATOM_untable,          1, onuntable),
-#ifdef O_PLMT
+#ifdef O_ENGINES
   GEVENT(PLEV_THREAD_START,     ATOM_thread_start,     1, onthreadstart),
   GEVENT(PLEV_THREAD_EXIT,      ATOM_thread_exit,      1, onthreadexit),
   LEVENT(PLEV_THIS_THREAD_EXIT, ATOM_this_thread_exit, 0, onthreadexit),
@@ -399,13 +403,13 @@ call_event_list(DECL_LD event_list *list, int argc, term_t argv)
     { if ( ev->function )
       { switch(argc)
 	{ case 0:
-	    rc = (*ev->function)(ev->closure.pointer);
+	    rc = (*(ev_func0)ev->function)(ev->closure.pointer);
 	    break;
 	  case 1:
-	    rc = (*ev->function)(ev->closure.pointer, argv+1);
+	    rc = (*(ev_func1)ev->function)(ev->closure.pointer, argv+1);
 	    break;
 	  case 2:
-	    rc = (*ev->function)(ev->closure.pointer, argv+1, argv+2);
+	    rc = (*(ev_func2)ev->function)(ev->closure.pointer, argv+1, argv+2);
 	    break;
 	  default:
 	    rc = FALSE;
@@ -628,7 +632,7 @@ PL_call_event_hook_va(pl_event_type ev, va_list args)
 			 ev == PLEV_BREAK          ? ATOM_true :
 			 ev == PLEV_NOBREAK        ? ATOM_false :
 			 ev == PLEV_RETRACTNOBREAK ? ATOM_retract :
-			                             ATOM_exist) &&
+						     ATOM_exist) &&
 	     PL_put_clref(av+2, cl) &&
 	     PL_put_intptr(av+3, offset) );
       break;
@@ -639,7 +643,7 @@ PL_call_event_hook_va(pl_event_type ev, va_list args)
       rc = PL_put_frame(av+1, fr);
       break;
     }
-#ifdef O_PLMT
+#ifdef O_ENGINES
     case PLEV_THREAD_START:
     case PLEV_THREAD_EXIT:
     { PL_thread_info_t *info = va_arg(args, PL_thread_info_t*);
