@@ -124,7 +124,7 @@ var_name_ptr(DECL_LD Word p, char *name)
   else
     iref = ((Word)p - (Word)gBase)*2;
 
-  Ssprintf(name, "_%lld", (int64_t)iref);
+  Ssprintf(name, "_%" PRIi64, (int64_t)iref);
 
   return name;
 }
@@ -620,7 +620,7 @@ putQuoted(int c, int quote, int flags, IOSTREAM *stream)
   } else
   { if ( !Putc(c, stream) )
       fail;
-    if ( c == quote || c == '\\' )	/* write '' or \\ */
+    if ( c == quote )	/* write '' */
     { if ( !Putc(c, stream) )
 	fail;
     }
@@ -1199,13 +1199,13 @@ format_float(double f, char *buf)
   return buf;
 }
 
-#ifdef O_GMP
+#ifdef O_BIGNUM
 #define writeMPZ(mpz, options) LDFUNC(writeMPZ, mpz, options)
 static int
 writeMPZ(DECL_LD mpz_t mpz, write_options *options)
 { char tmp[1024];
   char *buf;
-  size_t sz = mpz_sizeinbase(mpz, 10) + 2;
+  size_t sz = (mpz_sizeinbase(mpz, 2)*10)/3 + 10; /* log2(10)=3.322 */
   int rc;
 
   if ( sz <= sizeof(tmp) )
@@ -1243,7 +1243,7 @@ WriteNumber(Number n, write_options *options)
       sprintf(buf, "%" PRId64, n->value.i);
       return PutToken(buf, options->out);
     }
-#ifdef O_GMP
+#ifdef O_BIGNUM
     case V_MPZ:
       return writeMPZ(n->value.mpz, options);
     case V_MPQ:
@@ -1650,7 +1650,7 @@ writeTerm2(term_t t, int prec, write_options *options, int flags)
 	   PL_get_arg(1, t, class) )
       { if ( writeTerm(class, 1200, options, W_TAG) &&
 	     Putc('{', out) &&
-	     PL_for_dict(t, writeDictPair, options, DICT_SORTED) == 0 &&
+	     _PL_for_dict(t, writeDictPair, options, PL_FOR_DICT_SORTED) == 0 &&
 	     Putc('}', out) )
 	  return TRUE;
       }
@@ -1753,7 +1753,7 @@ writeTerm2(term_t t, int prec, write_options *options, int flags)
 	    { switch(writeAtom(functor, options))
 	      { case FALSE:
 		  fail;
-	        case TRUE_WITH_SPACE:
+		case TRUE_WITH_SPACE:
 		  TRY(Putc(' ', out));
 	      }
 	    }
@@ -2154,6 +2154,11 @@ PL_write_term(IOSTREAM *s, term_t term, int precedence, int flags)
   options.out	    = s;
   options.module    = MODULE_user;
 
+  if ( !(flags & (PL_WRT_CHARESCAPES|PL_WRT_NO_CHARESCAPES) ) )
+  { if ( true(options.module, M_CHARESCAPE) )
+      options.flags |= PL_WRT_CHARESCAPES;
+  }
+
   if ( (s=PL_acquire_stream(s)) )
   { PutOpenToken(EOF, s);			/* reset this */
     rc = writeTopTerm(term, precedence, &options);
@@ -2460,4 +2465,3 @@ BeginPredDefs(write)
   PRED_DEF("$put_quoted", 4, put_quoted_codes, 0)
   PRED_DEF("write_length", 3, write_length, 0)
 EndPredDefs
-
